from themer import ThemeActivator
import re
import os

class ACYLActivator(ThemeActivator):
    def icon_path(self):
        return os.path.join(os.environ['HOME'], '.icons/acyl')

    def primary_icon(self):
        return os.path.join(self.icon_path(), 'scalable/places/desktop.svg')

    def secondary_icon(self):
        return os.path.join(self.icon_path(), 'scalable/actions/add.svg')

    def extract_color_svg(self, filename):
        regex = re.compile('stop-color:(#[\da-zA-Z]{6})')
        with open(filename, 'r') as fh:
            for line in fh.readlines():
                match_obj = regex.search(line)
                if match_obj:
                    return match_obj.groups()[0]
        raise ValueError('Unable to determine icon color.')

    def activate(self):
        old_primary = self.extract_color_svg(self.primary_icon())
        old_secondary = self.extract_color_svg(self.secondary_icon())
        logger.debug('Old icon colors: {}, {}'.format(old_primary, old_secondary))

        # Walk the icons, updating the colors in each svg file.
        file_count = 0
        for root, dirs, filenames in os.walk(self.icon_path()):
            for filename in filenames:
                if not filename.endswith('.svg'):
                    continue
                path = os.path.join(root, filename)
                with open(path, 'r+') as fh:
                    contents = fh.read()
                    contents = contents.replace(old_primary, self.colors["primary"])
                    contents = contents.replace(old_secondary, self.colors["secondary"])
                    fh.seek(0)
                    fh.write(contents)
                    file_count += 1
        logger.info('Checked {} icon files'.format(file_count))



from themer.parsers import KmeansColorParser

from tempfile import mkstemp
import re
import requests
import os

class UnsplashColorParser(KmeansColorParser):
    check = 'unsplash.com/.*photo=[_\w\d]+$'

    def __init__(self, wallpaper, config, logger, k=16, bg='#0e0e0e', fg='#ffffff'):
        wallid = re.search("=([_\w\d]+)$", wallpaper).groups()[0]
        url = "http://unsplash.com/photos/{}/download".format(wallid)
        suffix = '.jpg'
        res = requests.get(url, stream=True)
        (dest, path) = mkstemp(suffix=suffix)
        for block in res.iter_content(1024):
            os.write(dest, block)
        os.close(dest)
        super(UnsplashColorParser, self).__init__(path, config, logger, k, bg, fg)

    def read(self, *args):
        return super(UnsplashColorParser, self).read(*args)



from themer import ThemeActivator
import os
try:
    import Image, ImageDraw
except ImportError:
    from PIL import Image, ImageDraw

HOME = os.environ['HOME']

try:
    import re
    screen_info = os.popen('xrandr').readlines()
    resolution_re = re.compile('\s+(\d+)x(\d+)\s+\d+\.\d+\*')
    for line in screen_info:
        match_obj = resolution_re.search(line)
        if match_obj:
            WIDTH, HEIGHT = map(int, match_obj.groups())
            break
except:
    WIDTH = 1440
    HEIGHT = 900
RATIO = float(WIDTH) / HEIGHT


class WallfixActivator(ThemeActivator):
    def hex_to_rgb(self, h):
        h = h.lstrip('#')
        return tuple(map(lambda n: int(n, 16), [h[i:i+2] for i in range(0, 6, 2)]))

    def new_wallpaper(self, w=1920, h=1200, filename='wallpaper.png'):
        rectangles = (
            # x1, y1, x2, y2 -- in percents
            ('red', [0, 30.0, 3.125, 72.5]), # LEFT
            ('green', [50, 0, 76.5625, 12.5]), # TOP
            ('yellow', [96.875, 30.0, 100, 72.5]), # RIGHT
            ('magenta', [23.4375, 25.0, 50, 30.0]), # MID TOP LEFT
            ('white', [23.4375, 30.0, 50, 72.5]), # MID LEFT
            ('magenta', [50, 30.0, 76.5625, 72.5]), # MID RIGHT
            ('white', [50, 72.5, 76.5625, 87.5]), # MID BOTTOM RIGHT
        )
        def fix_coords(coords):
            m = [w, h, w, h]
            return [int(c * .01 * m[i]) for i, c in enumerate(coords)]
        background = self.hex_to_rgb(self.colors['black'])
        image = Image.new('RGB', (w, h), background)
        draw = ImageDraw.Draw(image)
        for color, coords in rectangles:
            x1, y1, x2, y2 = fix_coords(coords)
            draw.rectangle([(x1, y1), (x2, y2)], fill=self.hex_to_rgb(self.colors[color]))
        image.save(os.path.join(self.theme_dir, filename), 'PNG')
        return image

    def crop_wallpaper(self, image):
        width, height = image.size
        ratio = float(width) / height

        if ratio > RATIO:
            # resize to match height, then crop horizontally from center
            new_width = int(HEIGHT * ratio)
            image.thumbnail((new_width, HEIGHT), Image.ANTIALIAS)
            offset = int((new_width - WIDTH) / 2)
            cropped = image.crop((offset, 0, offset + WIDTH, HEIGHT))
        elif ratio < RATIO:
            # resize to match width, then crop vertically from center
            new_height = int(WIDTH / ratio)
            image.thumbnail((WIDTH, new_height), Image.ANTIALIAS)
            offset = int((new_height - HEIGHT) / 2)
            cropped = image.crop((0, offset, WIDTH, offset + HEIGHT))
        else:
            image.thumbnail((WIDTH, HEIGHT), Image.ANTIALIAS)
            cropped = image

        dest_jpg = os.path.join(HOME, '.wallpaper.jpg')
        dest_png = os.path.join(HOME, '.wallpaper.png')

        cropped.save(dest_jpg, 'JPEG', quality=100)
        cropped.save(dest_png, 'PNG')

    def activate(self):
        wallpaper = None
        for filename in os.listdir(self.theme_dir):
            if filename.startswith('wallpaper'):
                wallpaper = Image.open(os.path.join(self.theme_dir, filename))
                break
        if not wallpaper:
            self.logger.info('No wallpaper found in {}, generating new one.'.format(self.theme_dir))
            wallpaper = self.new_wallpaper()

        self.logger.info('Setting new wallpaper')
        self.crop_wallpaper(wallpaper)



from themer import ColorParser, check_file_regex

import os
import re
import math
import yaml
import requests #urllib.request
import random
from tempfile import mkstemp

class CachedColorParser(ColorParser):
    check = check_file_regex('^colors\.yaml$')

    def read(self):
        with open(self.data) as fh:
            self.colors = yaml.load(fh)
        return self.colors

class SweylaColorParser(ColorParser):
    check = '^(sweyla)?[0-9]+$'

    def mapping(self):
        return {
            'bg': ['background', 'black', 'alt_black'],
            'fg': ['foreground', 'white'],
            'nf': 'red',  # name of function / method
            'nd': 'alt_red',  # decorator
            'nc': 'green',  # name of class
            'nt': 'alt_green', # ???
            'nb': 'yellow',  # e.g., "object" or "open"
            'c': 'alt_yellow',  # comments
            's': 'blue',  # string
            'mi': 'alt_blue',  # e.g., a number
            'k': 'magenta',  # e.g., "class"
            'o': 'alt_magenta', # operator, e.g "="
            'bp': 'cyan',  # e.g., "self" keyword
            'si': 'alt_cyan', # e.g. "%d"
            'se': 'alt_white',
            'support_function': 'underline'}

    def read(self):
        mapping = self.mapping()
        resp = requests.get(
            'http://sweyla.com/themes/textfile/sweyla{}.txt'.format(
                re.search("([0-9]+)",self.data).groups()[0]))
        contents = resp.text
        for line in contents.splitlines():
            key, value = line.split(':\t')
            if key in mapping:
                colors = mapping[key]
                if not isinstance(colors, list):
                    colors = [colors]
                for color in colors:
                    self.colors[color] = value
        return self.colors

# TODO: "lazy" loading somehow?
import colorsys
import itertools
try:
    import Image, ImageDraw
except ImportError:
    from PIL import Image, ImageDraw

try:
    import kmeans
except ImportError:
    global kmeans

    class PyKmeans:
        def ec_dist(self, a, b):
            return math.sqrt(
                sum((a[0][i] - b[0][i]) ** 2 for i in range(3)))

        def c_center(self, points):
            vals = [0.0 for i in range(3)]
            plen = 0
            for p in points:
                plen += p[1]
                for i in range(3):
                    vals[i] += p[0][i] * p[1]
            return ([(v/plen) for v in vals], 1)

        def kmeans(self, points, k, min_diff=1):
            print("""Falling back to python kmeans implementation.
            Consider installing 'kmeans' from PyPI for much faster image sampling""")
            clusters = [([p], p) for p in random.sample(points, k)]
            while True:
                plists = [[] for i in range(k)]
                for p in points:
                    smallest_distance = float('Inf')
                    for i in range(k):
                        distance = self.ec_dist(p, clusters[i][1])
                        if distance < smallest_distance:
                            smallest_distance = distance
                            idx = i
                    plists[idx].append(p)
                diff = 0
                for i in range(k):
                    old = clusters[i]
                    center = self.c_center(plists[i])
                    new = (plists[i], center)
                    clusters[i] = new
                    diff = max(diff, self.ec_dist(old[1], new[1]))
                if diff <= min_diff:
                    break
            return [map(int, c[1][0]) for c in clusters]

    kmeans = PyKmeans()

class KmeansColorParser(ColorParser):
    check = check_file_regex('\.(jpg|png|jpeg)$')

    def __init__(self, wallpaper, config, logger, k=16, bg='#0e0e0e', fg='#ffffff'):
        self.wallpaper = wallpaper
        self.config = config
        self.logger = logger
        self.bg = bg
        self.fg = fg
        self.k = k

    def _get_points_from_image(self, img):
        points = []
        w, h = img.size
        for count, color in img.getcolors(w * h):
            points.append((color, count))
        return points

    def rgb_to_hex(self, rgb):
        return '#{}'.format(''.join(('%02x' % int(p) for p in rgb)))

    def hex_to_rgb(self, h):
        h = h.lstrip('#')
        return tuple(map(lambda n: int(n, 16), [h[i:i+2] for i in range(0, 6, 2)]))

    def get_dominant_colors(self):
        img = Image.open(self.wallpaper)
        img.thumbnail((300, 300))  # Resize to speed up python loop.
        width, height = img.size
        points = self._get_points_from_image(img)
        rgbs = kmeans.kmeans(points, self.k)
        #rgbs = [map(int, c.center.coords) for c in clusters]
        return [self.rgb_to_hex(rgb) for rgb in rgbs]

    def normalize(self, hexv, minv=128, maxv=256):
        r, g, b = self.hex_to_rgb(hexv)
        h, s, v = colorsys.rgb_to_hsv(r / 256.0, g / 256.0, b / 256.0)
        minv = minv / 256.0
        maxv = maxv / 256.0
        if v < minv:
            v = minv
        if v > maxv:
            v = maxv
        rgb = colorsys.hsv_to_rgb(h, s, v)
        return self.rgb_to_hex(map(lambda i: i * 256, rgb))

    def read(self):
        colors = self.get_dominant_colors()
        color_dict = {
            'background': self.bg,
            'foreground': self.fg}
        for i, color in enumerate(itertools.cycle(colors)):
            if i == 0:
                color = self.normalize(color, minv=0, maxv=32)
            elif i == 8:
                color = self.normalize(color, minv=128, maxv=192)
            elif i < 8:
                color = self.normalize(color, minv=160, maxv=224)
            else:
                color = self.normalize(color, minv=200, maxv=256)
            color_dict['color%d' % i] = color
            if i == 15:
                break
        mapping = self.mapping()
        translated = {}
        for k, v in color_dict.items():
            translated[mapping[k]] = v
        return translated

class WallhavenColorParser(KmeansColorParser):
    check = 'wallhaven.cc/wallpaper/[0-9]+$'

    def __init__(self, wallpaper, config, logger, k=16, bg='#0e0e0e', fg='#ffffff'):
        wallid = re.search("([0-9]+)", wallpaper).groups()[0]
        suffix = '.jpg'
        res = requests.get('http://wallpapers.wallhaven.cc/wallpapers/full/wallhaven-{}.jpg'.format(wallid), stream=True)

        if not res.ok:
            res = requests.get('http://wallpapers.wallhaven.cc/wallpapers/full/wallhaven-{}.png'.format(wallid), stream=True)
            suffix = '.png'

        (dest, path) = mkstemp(suffix=suffix)

        for block in res.iter_content(1024):
            os.write(dest, block)
        os.close(dest)

        super(WallhavenColorParser, self).__init__(path, config, logger, k, bg, fg)

    def read(self, *args):
        res = super(WallhavenColorParser, self).read(*args)
        return res



#!/usr/bin/env python
# vim:fileencoding=utf-8:noet
from __future__ import unicode_literals
import os
import sys

from setuptools import setup, find_packages
from setuptools.command.install import install

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.md'), 'rb').read().decode('utf-8')
except IOError:
    README = ''

class new_install(install):
    def run(self):
        print( self.root )
        install.run(self) # invoke original install

        root = self.root or '/'
        self.mkpath(os.path.join(root, 'usr/share/themer'))
        self.mkpath(os.path.join(root, 'usr/share/fish/completions'))
        self.copy_tree('data/default', os.path.join(root, 'usr/share/themer/default'))
        self.copy_file('data/fish/themer.fish', os.path.join(root, 'usr/share/fish/completions/'))

setup(
    name='Themer',
    version='1.7',
    description='Themer is a colorscheme generator and manager for your desktop.',
    long_description=README,
    author='Charles Leifer, Sol Bekic',
    author_email='s0lll0s@blinkenshell.org',
    url='https://github.com/S0lll0s/themer',
    scripts=['scripts/themer'],
    license='MIT',
    keywords='wm colorscheme color theme wallpaper',
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    install_requires=['pyyaml','jinja2','pillow', 'requests'],
    cmdclass=dict(install=new_install)
)



import os.path
import re

def check_file_regex(regex):
    """Builds a function that checks wheter a filename matches 'regex' and corresponds to an existing file.

    For use as a ColorParser's 'check' attribute."""
    return staticmethod(lambda src: re.search(regex, src, re.IGNORECASE) and os.path.isfile(src))

class ThemeActivator(object):
    def __init__(self, theme_name, theme_dir, logger):
        from themer.parsers import CachedColorParser

        self.theme_name = theme_name
        self.theme_dir  = theme_dir
        self.logger     = logger
        self.colors     = CachedColorParser(os.path.join(self.theme_dir, 'colors.yaml'), None, logger).read()

    def activate(self):
        pass

class ColorParser(object):
    check = check_file_regex('(.*)')

    # Colors look something like "*color0:  #FF0d3c\n"
    color_re = re.compile('.*?(color[^:]+|background|foreground):\s*(#[\da-z]{6})')

    def __init__(self, data, config, logger):
        self.data = data
        self.config = config
        self.logger = logger
        self.colors = {}
        self.wallpaper = None

    def mapping(self):
        return {
            'background': 'background',
            'foreground': 'foreground',
            'color0': 'black',
            'color8': 'alt_black',
            'color1': 'red',
            'color9': 'alt_red',
            'color2': 'green',
            'color10': 'alt_green',
            'color3': 'yellow',
            'color11': 'alt_yellow',
            'color4': 'blue',
            'color12': 'alt_blue',
            'color5': 'magenta',
            'color13': 'alt_magenta',
            'color6': 'cyan',
            'color14': 'alt_cyan',
            'color7': 'white',
            'color15': 'alt_white',
            'colorul': 'underline'}

    def read(self):
        color_mapping = self.mapping()

        with open(self.data) as fh:
            for line in fh.readlines():
                if line.startswith('!'):
                    continue
                match_obj = self.color_re.search(line.lower())
                if match_obj:
                    var, color = match_obj.groups()
                    self.colors[color_mapping[var]] = color

        if len(self.colors) < 16:
            self.logger.warning(
                'Error, only {} colors were read when loading color file'
                .format(len(self.colors)))
        return self.colors
